<?php

namespace Drupal\peakhour_purge;


use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Messenger\Messenger;
use GuzzleHttp\ClientInterface;
use Psr\Log\LoggerInterface;
use Symfony\Component\HttpFoundation\RequestStack;

class API
{

  const FLUSH_ALL = 'all';

  /**
   * Peakhour API Url.
   *
   * @var string
   */
  protected $peakhourUrl;

  /**
   * Peakhour API Key.
   *
   * @var string
   */
  protected $apiKey;

  /**
   * Domain Name
   *
   * @var string
   */
  protected $domain;

  /**
   * The Peakhour logger channel.
   *
   * @var \Psr\Log\LoggerInterface
   */
  protected $logger;

  /**
   * The Peakhour logger channel.
   *
   * @var \GuzzleHttp\ClientInterface;
   */
  protected $httpClient;

  /**
   * The Peakhour logger channel.
   *
   * @var boolean
   */
  protected $enabled;

  /**
   * Connect timeout.
   *
   * @var string
   */
  protected $connectTimeout;


  public function __construct(ConfigFactoryInterface $config_factory, $connect_timeout, ClientInterface $http_client, LoggerInterface $logger) {

    $config = $config_factory->get('peakhour_purge.settings');
    $this->apiKey = $config->get('api_key');
    $this->domain = $config->get('domain_name');
    $this->peakhourUrl = $config->get('api_url');
    $this->enabled = $config->get('enabled');
    $this->httpClient = $http_client;
    $this->connectTimeout = $connect_timeout;
    $this->logger = $logger;
  }

  /**
   * @return string
   */
  public function getPeakhourUrl(): mixed
  {
    return $this->peakhourUrl;
  }

  /**
   * @param string $peakhourUrl
   */
  public function setPeakhourUrl(mixed $peakhourUrl): void
  {
    $this->peakhourUrl = $peakhourUrl;
  }

  /**
   * @return string
   */
  public function getApiKey(): mixed
  {
    return $this->apiKey;
  }

  /**
   * @param string $apiKey
   */
  public function setApiKey(mixed $apiKey): void
  {
    $this->apiKey = $apiKey;
  }

  /**
   * @return string
   */
  public function getDomain(): mixed
  {
    return $this->domain;
  }

  /**
   * @param string $domain
   */
  public function setDomain(mixed $domain): void
  {
    $this->domain = $domain;
  }

  /**
   * @return LoggerInterface
   */
  public function getLogger(): LoggerInterface
  {
    return $this->logger;
  }

  /**
   * @param LoggerInterface $logger
   */
  public function setLogger(LoggerInterface $logger): void
  {
    $this->logger = $logger;
  }

  /**
   * @return ClientInterface
   */
  public function getHttpClient(): ClientInterface
  {
    return $this->httpClient;
  }

  /**
   * @param ClientInterface $httpClient
   */
  public function setHttpClient(ClientInterface $httpClient): void
  {
    $this->httpClient = $httpClient;
  }

  /**
   * @return bool
   */
  public function isEnabled(): mixed
  {
    return $this->enabled;
  }

  /**
   * @param bool $enabled
   */
  public function setEnabled(mixed $enabled): void
  {
    $this->enabled = $enabled;
  }



  /**
   *
   *  purge all public cache of this site
   *
   * @since   1.0.0
   */
  public function purgeAllPublic()
  {
    $resourcesUrl = $this->getResourcesUrl();
    $body = $this->makeRequestBody([], self::FLUSH_ALL, true);
    return $this->doRequest($resourcesUrl, 'DELETE', $body);
  }


  public function purgeTags($tags)
  {
    $this->logger->debug("flushing: " . implode(",", $tags));
    $response = $this->doRequest($this->getTagsUrl(), 'DELETE', json_encode(array('tags' => $tags)));
    return $response;
  }

  function purgeUrls($urls)
  {
    $resourcesUrl = $this->getResourcesUrl();
    $body = $this->makeRequestBody($urls);
    return $this->doRequest($resourcesUrl, 'DELETE', $body);
  }

  public function testConnection()
  {
    $resourcesUrl = $this->getResourcesUrl();
    return $this->doRequest($resourcesUrl, 'OPTIONS', null);
  }

  function makeRequestBody($urls, $flushType = self::FLUSH_ALL, $purgeAll = false)
  {
    $paths = array();

    foreach ($urls as $url) {
      $parsed = parse_url($url);
      $path = (isset($parsed['path']) ? $parsed['path'] : null);
      if (empty($path)) {
        $path = '/';
      }
      if (isset($parsed['query'])) {
        $path = $path . '?' . $parsed['query'];
      }
      array_push($paths, $path);

    }
    $paths = array_values(array_unique($paths));

    return json_encode(array('paths' => $paths, 'flush_type' => $flushType, 'purge_all' => $purgeAll));
  }

  function getTagsUrl()
  {
    if (is_null($this->domain) || is_null($this->peakhourUrl)) {
      return "";
    }
    return $this->peakhourUrl . 'domains/' . $this->domain . '/services/rp/cdn/tag';
  }

  function getResourcesUrl($domain = null)
  {
    if (is_null($this->domain) || is_null($this->peakhourUrl)) {
      return "";
    }
    return $this->peakhourUrl . 'domains/' . $this->domain . '/services/rp/cdn/resources';
  }

  function doRequest($url, $method, $body = null)
  {
    if (is_null($this->apiKey) || is_null($this->domain) || is_null($this->peakhourUrl)) {
      return array('success' => false, 'error' => "You need to make sure you've provided your API Key, peakhour api url, and domain in the Peakhour module settings");
    }
    $headers = array(
      'Accept' => 'application/json',
      'Authorization' => 'Bearer ' . $this->apiKey,
      'Content-Type' => 'application/json',
    );

    try {


      if (!is_null($body)) {
        $response = $this->httpClient->request($method, $url, ['headers' => $headers, 'body' => $body, 'connect_timeout' => $this->connectTimeout]);
      } else {
        $response = $this->httpClient->request($method, $url, ['headers' => $headers, 'connect_timeout' => $this->connectTimeout]);
      }
      $this->logger->debug("returning");
      return array('success' => true, 'error' => null, 'body' => $response->getBody());
    } catch (\Exception $e) {
      $this->logger->debug("Failed $method to $url: " . $e->getMessage());
      return array('success' => false, 'error' => $e->getMessage());
    }
  }
}
